from datetime import datetime
from enum import Enum, IntEnum


class BaseClass:
    pass


class ApcStatus(Enum):
    OK = 0
    IN_PROGRESS = 1
    UPLOAD = 2


class ApcCategory(Enum):
    Unknown = 0
    Clean = 1
    Malware = 2
    AdSpy = 3
    Aware = 4
    Appl = 5
    Bat = 6
    Bdc = 7
    Bds = 8
    Boo = 9
    Ddos = 10
    Dial = 11
    Dos = 12
    Dr = 13
    Eml = 14
    Exp = 15
    Game = 16
    Html = 17
    Irc = 18
    Java = 19
    Joke = 20
    Js = 21
    Kit = 22
    Linux = 23
    Osx = 24
    Perl = 25
    Pfs = 26
    Phish = 27
    Php = 28
    Rkit = 29
    Spr = 30
    Swf = 31
    Symbos = 32
    Tr = 33
    Unix = 34
    Vbs = 35
    W32 = 36
    W64 = 37
    Worm = 38
    W95 = 39
    W2000 = 40
    Heur = 41
    Pck = 42
    Android = 43
    HiddenText = 44
    W2000M = 45
    W97M = 46
    Pua = 47
    Cln = 48
    Vba = 49


class ClassificationDescription(IntEnum):
    # This represents an uninitialized value.
    Unset = 0
    # The classification was manually set.
    ManuallySet = 1
    # The classification was performed by the algorithm,
    # and there was a clear winner.
    # In this case, all votes were cast to one item.
    ClearWinner = 2
    # The classification was performed by the algorithm,
    # and there was a majority winner.
    # In this case, the winning item had at least two votes
    # and all other items had no more than one vote.
    MajorityWinner = 3
    # The classification was performed by the algorithm,
    # and there was a contested winner.
    # In this case, the winning item had more votes than any other item,
    # but there were multiple votes cast for other items.
    ContestedWinner = 4
    # The classification was performed by the algorithm, and there was a tie.
    # The tie was a result of the number of scanners.
    # In this case, the item with the largest score would have been chosen.
    TiedWinnerByNumberOfScanners = 5
    # The classification was performed by the algorithm, and there was a tie.
    # The tie was a result of the score.
    # In this case, the item that had the most trusted scanner vote for it
    # is selected as the winner.
    # It is possible that there could also be a tie by the  number of scanners.
    TiedWinnerByScore = 6


class SignatureStatus(Enum):
    Unknown = 0
    SignedAndVerified = 1
    NotSigned = 2
    SignedNotVerified = 3
    SignedNotTrusted = 4
    SignedNotAllowed = 5
    InvalidCertificate = 6


class SignatureInfo(BaseClass):
    def __init__(
        self,
        signed_by: str = None,
        authority: str = None,
        signature_statue: SignatureStatus = None,
    ):
        self.signed_by = signed_by
        self.authority = authority
        self.signature_status = signature_statue


class ClassificationDetail(BaseClass):
    def __init__(
        self,
        num_scanner_votes: int = 0,
        result: str = "",
        description: [ClassificationDescription, None] = None,
    ):
        self.num_scanner_votes = num_scanner_votes
        self.result = result
        self.description = description


class Reputation(BaseClass):
    def __init__(
        self,
        trust: int = 0,
        threat: int = 0,
        trust_locked: bool = False,
        threat_locked: bool = False,
        *args,
        **kwargs
    ):
        self.trust = trust
        self.threat = threat
        self.trust_locked = trust_locked
        self.threat_locked = threat_locked


class ScanDetails(BaseClass):
    def __init__(
        self,
        positives: int = 0,
        total_scans: int = 0,
        first_scanned_date: datetime = None,
        last_scanned_date: datetime = None,
        positive_scanners: list = None,
    ):
        self.positives = positives
        self.total_scans = total_scans
        self.first_scanned_date = first_scanned_date
        self.last_scanned_date = last_scanned_date
        self.positive_scanners = positive_scanners or []


class MfgVersionResponse(BaseClass):
    def __init__(
        self,
        mfg_version_id: int = None,
        version: str = None,
        date_created: datetime = None,
        *args,
        **kwargs
    ):
        self.mfg_version_id = mfg_version_id or kwargs.get("id") or 0
        self.version = version
        self.date_created = date_created


class Mfg(BaseClass):
    def __init__(
        self,
        signature: [str, None] = None,
        tags: [list, None] = None,
        classification_scanners: [list, None] = None,
        platform: [ClassificationDetail, None] = None,
        malware_group: [ClassificationDetail, None] = None,
        malware_family: [ClassificationDetail, None] = None,
        locked: bool = False,
    ):
        self.signature = signature
        self.tags = tags or []
        self.classification_scanners = classification_scanners or []
        self.platform = platform
        self.malware_group = malware_group
        self.malware_family = malware_family
        self.locked = locked

    def minimize(self):
        if (
            self.platform.description is None
            and self.platform.num_scanner_votes is None
            and self.platform.description is None
        ):
            self.platform = None
        if (
            self.malware_group.description is None
            and self.malware_group.num_scanner_votes is None
            and self.malware_group.description is None
        ):
            self.malware_group = None
        if (
            self.malware_family.description is None
            and self.malware_family.num_scanner_votes is None
            and self.malware_family.description is None
        ):
            self.malware_family = None


class Apc(BaseClass):
    def __init__(
        self,
        status: [ApcStatus, None] = None,
        known: bool = False,
        category: [ApcCategory, None] = None,
        signature: [str, None] = None,
        ttl: [int, None] = None,
    ):
        self.status = status
        self.known = known
        self.category = category
        self.signature = signature
        self.ttl = ttl


class ReputationFeedDto(BaseClass):
    def __init__(
        self,
        sha256: [str, None] = None,
        md5: [str, None] = None,
        sha1: [str, None] = None,
        tlsh: [str, None] = None,
        reputation: [Reputation, None] = None,
        file_name: [str, None] = None,
        cb_first_seen: [datetime, None] = None,
        feed_date: [datetime, None] = None,
        scan_details: [ScanDetails, None] = None,
        sources: [list, None] = None,
        mfg: [Mfg, None] = None,
        signature_info: [SignatureInfo, None] = None,
    ):
        self.sha256 = sha256
        self.md5 = md5
        self.sha1 = sha1
        self.tlsh = tlsh
        self.reputation = reputation
        self.file_name = file_name
        self.cb_first_seen = cb_first_seen
        self.feed_date = feed_date
        self.scan_details = scan_details
        self.sources = sources or []
        self.mfg = mfg
        self.signature_info = signature_info

    def minimize(self):
        if self.reputation is None or (
            self.reputation.threat is None
            and self.reputation.trust is None
            and self.reputation.trust_locked
        ):
            self.reputation = None
        if (
            self.scan_details.first_scanned_date is None
            and self.scan_details.last_scanned_date is None
            and (
                self.scan_details.positive_scanners is None
                or len(self.scan_details.positive_scanners) == 0
            )
            and (
                self.scan_details.positives is None or self.scan_details.positives == 0
            )
            and (
                self.scan_details.total_scans is None
                or self.scan_details.total_scans == 0
            )
        ):
            self.scan_details = None
        if self.mfg:
            self.mfg.minimize()
        if self.signature_info is None or (
            self.signature_info.signature_status is None
            and self.signature_info.authority is None
            and self.signature_info.signed_by is None
        ):
            self.signature_info = None
        self.sources = list(set(map(lambda x: x.lower(), self.sources)))


class HashReputationUpdateDto(BaseClass):
    def __init__(
        self,
        md5: [str, None] = None,
        sha1: [str, None] = None,
        sha256: [str, None] = None,
        filename: [str, None] = None,
        size: int = 0,
        comment: [str, None] = None,
        trust: float = 0.0,
        threat: int = 0,
        malware_family: [str, None] = None,
        malware_group: [str, None] = None,
        platform: [str, None] = None,
        username: [str, None] = None,
        *args,
        **kwargs
    ):
        self.md5 = md5
        self.sha1 = sha1
        self.sha256 = sha256
        self.filename = filename
        self.size = size
        self.comment = comment
        self.trust = trust
        self.threat = threat
        self.malware_family = malware_family
        self.malware_group = malware_group
        self.platform = platform
        self.username = username


class MfgReputationData(BaseClass):
    def __init__(
        self,
        sha256: [str, None] = None,
        md5: [str, None] = None,
        sha1: [str, None] = None,
        scan_details: [ScanDetails, None] = None,
        sources: [list, None] = None,
        mfg: [Mfg, None] = None,
        threat: int = 0,
        threat_locked: bool = False,
        date_created: [datetime, None] = None,
    ):
        self.sha256 = sha256
        self.md5 = md5
        self.sha1 = sha1
        self.scan_details = scan_details
        self.sources = sources or []
        self.mfg = mfg
        self.threat = threat
        self.threat_locked = threat_locked
        self.date_created = date_created


class Bit9ReputationData(BaseClass):
    def __init__(
        self,
        sha256: [str, None] = None,
        md5: [str, None] = None,
        sha1: [str, None] = None,
        tlsh: [str, None] = None,
        reputation: [Reputation, None] = None,
        file_name: [str, None] = None,
        cb_first_seen: [datetime, None] = None,
        sources: [list, None] = None,
        signature_info: [SignatureInfo, None] = None,
        apc: [Apc, None] = None,
        scan_details: [ScanDetails, None] = None,
        feed_date: [datetime, None] = None,
        *args,
        **kwargs
    ):
        self.sha256 = sha256
        self.md5 = md5
        self.sha1 = sha1
        self.tlsh = tlsh
        self.reputation = reputation
        self.file_name = file_name
        self.cb_first_seen = cb_first_seen
        self.sources = sources or []
        self.signature_info = signature_info
        self.apc = apc
        self.scan_details = scan_details
        self.feed_date = feed_date


class ProcessingCheckpoints(BaseClass):
    def __init__(
        self,
        last_processed_id: [int, None] = None,
        newest_update_id: [int, None] = None,
    ):
        self.last_processed_id = last_processed_id
        self.newest_update_id = newest_update_id


class ProcessDetails(BaseClass):
    def __init__(
        self,
        mfg_sha256s: [list[str], None] = None,
        mfg_updates: [int, None] = None,
        mfg_last_processed_id: [int, None] = None,
    ):
        self.mfg_sha256s = mfg_sha256s or []
        self.mfg_updates = mfg_updates
        self.mfg_last_processed_id = mfg_last_processed_id


class BaseSha256Response(BaseClass):
    def __init__(self, sha256: [str, None] = None):
        self.sha256 = sha256


class MfgInfoResponse(BaseSha256Response):
    def __init__(
        self,
        sha256: [str, None] = None,
        sha1: [str, None] = None,
        md5: [str, None] = None,
        additional_info: [str, None] = None,
        platform_num_scanner_votes: int = 0,
        platform_result: [str, None] = None,
        platform_description: [ClassificationDescription, None] = None,
        malware_group_num_scanner_votes: int = 0,
        malware_group_result: [str, None] = None,
        malware_group_description: [ClassificationDescription, None] = None,
        malware_family_num_scanner_votes: int = 0,
        malware_family_result: [str, None] = None,
        malware_family_description: [ClassificationDescription, None] = None,
        date_created: [datetime, None] = None,
        is_malicious: bool = False,
        mfg_locked: bool = False,
        threat: int = 0,
        threat_locked: bool = False,
    ):
        super().__init__(sha256)
        self.sha1 = sha1
        self.md5 = md5
        self.additional_info = additional_info
        self.platform_num_scanner_votes = platform_num_scanner_votes
        self.platform_result = platform_result
        self.platform_description = platform_description
        self.malware_group_num_scanner_votes = malware_group_num_scanner_votes
        self.malware_group_result = malware_group_result
        self.malware_group_description = malware_group_description
        self.malware_family_num_scanner_votes = malware_family_num_scanner_votes
        self.malware_family_result = malware_family_result
        self.malware_family_description = malware_family_description
        self.date_created = date_created
        self.is_malicious = is_malicious
        self.mfg_locked = mfg_locked
        self.threat = threat
        self.threat_locked = threat_locked


class Sha256EnumerableStringResponse(BaseSha256Response):
    def __init__(self, sha256: [str, None] = None, values: [list[str], None] = None):
        super().__init__(sha256)
        self.values = values or []


class SourcesResponse(BaseSha256Response):
    def __init__(self, sha256: [str, None] = None, sources: [list[str], None] = None):
        super().__init__(sha256)
        self.sources = sources or []


class ScannedDatesResponse(BaseSha256Response):
    def __init__(
        self,
        sha256: [str, None] = None,
        first_scanned_date: [datetime, None] = None,
        last_scanned_date: [datetime, None] = None,
    ):
        super().__init__(sha256)
        self.first_scanned_date = first_scanned_date
        self.last_scanned_date = last_scanned_date


class TotalScansResponse(BaseSha256Response):
    def __init__(self, sha256: [str, None] = None, total_scans: int = 0):
        super().__init__(sha256)
        self.total_scans = total_scans
